
// Kommando SS, 1..6
// Ss,2<13>

//------------------------------------------------------------------------------
// Programmed by Christof Ermer
//------------------------------------------------------------------------------

#define SOFTWARE	"StepMotor"
#define HARDWARE	"Arduino-UNO ATMega328"
#define DATUM		"09.03.2016 12:00"
#define AUTOR		"* Christof Ermer *"
#define IHAVENOBRAIN	"I have no brain"	
#define USEYOURS		"Use your own"

#include <avr/sfr_defs.h>
#include <avr\io.h>
#include <util\delay.h>
#include <avr/wdt.h>
#include <avr\interrupt.h>
#include <stdlib.h> //itoa ultoa
#include <avr\pgmspace.h> // <--#include <progmem.h>
#include <math.h>
#include "main.h"
#include "uart.h"
#include "Befehle.h"
#include "ServiceFunc.h"
#include "debug.h"
#include "max7219.h"

// GLOBALE VARIABLEN 
volatile uint32_t gu32_Ticks;	 //  T2 TimerCounter
volatile uint8_t gu8Status;
volatile uint8_t gu8SM_Status;
int32_t	gi32SM_Cnt;

PGM_P gppmgPattern;
uint16_t gu16Parser;  // *( Parser + nn )
uint8_t gu8PatternSize; // Achtung eine 0 muss im Feld sein

#ifdef STATE_MACHINE_USED
volatile uint8_t gu8StateMach;
#endif

PGM_P gppmPattern;  //POINTER auf Flash
uint16_t gu16Parser;  // *( Parser + nn )
uint8_t gu8PatternSize; // Achtung eine 0 muss im Feld sein


const char MCA_ARDUINO_SM[] PROGMEM = {0x08,0x0C,0x04,0x06,0x02,0x03,0x01,0x09};	//CONST ARRAY
const char MCA_STARTREK[] PROGMEM = {0x81,0x42,0x24,0x18,0x24,0x42,0};
const char MCA_FULLSTEPS[] PROGMEM = {0x3A,0x39,0x35,0x36,0};	//CONST ARRAY
const char MCA_HALFSTEPS[] PROGMEM = {0x3A,0x28,0x39,0x11,0x35,0x24,0x36,0x12};	//CONST ARRAY

uint16_t gu16SOllFrequenz;
uint16_t gu16T1_Comp;

// ***** ***********************************************************
ISR( TIMER1_COMPA_vect )// signal handler	/#define SIG_OUTPUT_COMPARE1A	_VECTOR(6)
// ***** ***********************************************************
{
volatile static uint8_t u8Parser;
if( gu8SM_Status & MOTOR_DIRECTION )
	{
	SM_PORTX = pgm_read_byte_near( (PGM_P)gppmPattern + u8Parser++ );
	gi32SM_Cnt++;
	}	
else
	{
	SM_PORTX = pgm_read_byte_near( (PGM_P)gppmPattern + u8Parser-- );
	gi32SM_Cnt--;
	};
u8Parser %= gu8PatternSize;
};


// *********************************
void StartTimer_1(uint16_t u16Frq)
// *********************************
{
TCCR1B = 0;
TCNT1 = 0;
gu8SM_Status &= ~MOTOR_ON;
gu8SM_Status &= ~RAMPE_AKTIVE;
gu16SOllFrequenz = u16Frq;

if( u16Frq )
	{
	if( gu8SM_Status & RAMPE_ENABLED )
		{
		gu16T1_Comp = round( F_CPU / 1024.0 / gu16SOllFrequenz );
		OCR1A = round( 16000000UL / 1024.0 / 100 ); // Starte langsam
		gu8SM_Status |= RAMPE_AKTIVE;
		}
	else
		{
		OCR1A = round( 16000000UL / 1024.0 / gu16SOllFrequenz );
		};
	
	TIMSK1 = _BV(OCIE1A); //Interrupt
	TCCR1B = _BV( WGM12) | _BV( CS12) | _BV(CS10); //CTC + /1024
	gu8SM_Status |= MOTOR_ON;	
	sei();
	};
};


//Experiment kompilere folgende Zeile MIT UND OHNE PROGMEM

//So ist es falsch , weil es RAM verbraucht  !! Jedes einezlne kostbare Byte
//const char MCA_DISCO[] =    // VORSICHT BRAUCHT RAM


// Interrupt Section
// ********************
ISR( TIMER2_OVF_vect )    //Dieser Vector steht in der iom328p.h
// ********************
{ 	// Interrupt Variablen mssen "volatile" deklariert sein
volatile static uint32_t u32Merke_T;
volatile static uint32_t u32PWM_T;
//hochzhlen ? 244*/Sekunde
gu32_Ticks++;

if( ( (gu32_Ticks) - u32Merke_T) > TICK_500MS )
	{
	u32Merke_T = gu32_Ticks;	//neue Zeit merken
	gu8Status |= STATUS_TICK_EVENT;
	};
	
if( ( (gu32_Ticks) - u32PWM_T) > 10 )
	{
	u32PWM_T = gu32_Ticks;	//neue Zeit merken
	gu8Status |= STATUS_RAMPETIMER_FLAG;
	};	
// !! 244*/ Sekunde	
sei();
UART_TXRX_Check();  //langsam, aber sehen ob was kommt am UART
};



// ****************************
void Show_Byte(uint8_t u8Val )
// ****************************
{
//Bits an richtige Positionen schieben, wenn ntig!!
#define PORTB_LIGHT_MASKE 0x3F // 0b00111111
#define PORTD_LIGHT_MASKE 0xC0 // 0b11000000
DDRB |= PORTB_LIGHT_MASKE;
DDRD |= PORTD_LIGHT_MASKE;

PORTB &= ~PORTB_LIGHT_MASKE;
PORTB |= u8Val & PORTB_LIGHT_MASKE;
//( (u8Val &PORTD_LIGHT_MASKE) >> 2)
PORTD &= ~PORTD_LIGHT_MASKE;
PORTD |= u8Val & PORTD_LIGHT_MASKE;
};



#ifdef STATE_MACHINE_USED
// ********************************
void CheckStateMach(void)
// ********************************
{
switch( gu8StateMach )
	{
	case STATEMACH_NIX: 
		{
		break;
		};
	
	}; //switch
};
#endif



// *************
int main(void)
// *************
{

wdt_reset();   //WATCHDOG!
wdt_enable(5); //500mS Totmannknopf

uart_init( UART_BAUD_SELECT_DOUBLE_SPEED(UART_BAUD_RATE, F_CPU) );
sei(); 
ResetRxBuff();

//TickCounter Starten
TCCR2B = (1 << CS22) | (1 << CS21);	//Vorteiler //256 einschalten
TIMSK2 = (1 << TOIE2);	//Interrrupt aktivieren

sei();
uart_puts_p( PSTR( HARDWARE ));uart_putc(13); 
uart_puts_p(  PSTR( SOFTWARE ) );uart_putc(13);
uart_puts_p( PSTR( DATUM )  );uart_putc(13); 
uart_puts_p( PSTR( AUTOR ) );uart_putc(13); 
uart_puts_p( PSTR( IHAVENOBRAIN ) );uart_putc(13); 
uart_puts_p( PSTR( USEYOURS ) );uart_putc(13); 

//gu8Status |= STATUS_TICK_MSG_FLAG;
//gu8Status =0;
//MAX7219_Init();
//MAX7219_Clear ();

//gu8StateMach = ; // siehe "serviceFUnc.c"
//CheckStateMach();

//gppmPattern = MCA_FULLSTEPS;
gppmPattern = MCA_HALFSTEPS;

//gu8PatternSize = 4;
gu8PatternSize = 8;

SM_PORT_INIT(); // 6 Bit PORTB 

gu16SOllFrequenz = 400;

gu8SM_Status |= RAMPE_ENABLED;
StartTimer_1( gu16SOllFrequenz );

/*
OCR1A = round( F_CPU / 1024 / 100 );  //quarz/Vorteiler/ FRQ
TCNT1 = 0;
TIMSK1 |= _BV( OCIE1A );
//am schluss wird eingeschaltet
TCCR1B = _BV( WGM12 ) | _BV( CS12 ) | _BV( CS10 );	//CTC + /1024
sei()
*/

for(;;) 
	{
	sei();
	wdt_reset();	//WATCHDOG!

	if( gu8Status & STATUS_RAMPETIMER_FLAG )  // schnellere Aufruf
		{
		gu8Status &= ~STATUS_RAMPETIMER_FLAG;
		
Toggle_Bit(1);				
		if( (gu8SM_Status & MOTOR_ON)	&&
			(gu8SM_Status & RAMPE_ENABLED) &&
			(gu8SM_Status & RAMPE_AKTIVE)
		)
			{
			if( OCR1A > gu16T1_Comp)
				{
Toggle_Bit(2);				
				OCR1A -= 1;
				TCNT1 = 0;
				gu8SM_Status &= ~SOLL_FRQ_OK;
				}
			else
				{
Toggle_Bit(3);				
				OCR1A = gu16T1_Comp;
				TCNT1 = 0;				
				gu8SM_Status &= ~RAMPE_AKTIVE;
				gu8SM_Status |= SOLL_FRQ_OK;				
				};
			};
		};		

	if( gu8Status & STATUS_TICK_EVENT )
		{
		gu8Status &= ~STATUS_TICK_EVENT;		
		
		//PORTB ^= LED1_BLINK;	// ab jetzt langsame ~1Hz Action				
		if( gu8Status & STATUS_TICK_MSG_FLAG ) //TO/TF
			{
//			uart_puts_p( PSTR("PWM=") );
//			uart_puts( ultoa( OCR0A, gcaStr, 10 ) );
			//MAX7219_Write(2, OCR0A);
			uart_putc( 13 ); //CR
			};
			
		};
//CheckStateMach();
//Toggle_Bit(1);
	};
}; // main
