

/*
PWM & Mnemonic
OCR1A:
16=~1mS
24 fr ~1,5mS
32=2mS
*/

//------------------------------------------------------------------------------
// Programmed by Christof Ermer
//------------------------------------------------------------------------------

#define SOFTWARE	"Knigt Rider"
#define HARDWARE	"Arduino-UNO ATMega328"
#define DATUM		"31.03.20175 12:00"
#define AUTOR		"* Christof Ermer *"
#define IHAVENOBRAIN	"I have no brain"	
#define USEYOURS		"Use your own"

#include <avr/sfr_defs.h>
#include <avr\io.h>
#include <util\delay.h>
#include <avr/wdt.h>
#include <avr\interrupt.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <avr\pgmspace.h> // <--#include <progmem.h>
#include "main.h"
#include "uart.h"
#include "Befehle.h"
#include "debug.h"
#include "MAX543.h"
#include "OutPort74-595.h"

// GLOBALE VARIABLEN 
volatile uint32_t gu32_Ticks;	 //  T2 TimerCounter
volatile uint8_t gu8Status;

// Interrupt Section
// ********************
ISR( TIMER2_OVF_vect )    //Dieser Vector steht in der iom328p.h
// ********************
{ 	// Interrupt Variablen mssen "volatile" deklariert sein
volatile static uint32_t u32MerkeT;
Toggle_Bit(1);

//hochzhlen ? 244*/Sekunde
if( ( (gu32_Ticks++) - u32MerkeT) > 24 )
	{
	u32MerkeT = gu32_Ticks;	//neue Zeit merken
	gu8Status |= STATUS_TICK_EVENT;
	};
// !! 244*/ Sekunde	
UART_TXRX_Check();  //langsam, aber sehen ob was kommt am UART
};



// *************************************
void Timer0_PWM( uint8_t u8PwmRange)
// *************************************
{  // 8BIT TIMER T0   // #define  _BV(x) (1<<(x))
OCR0A = u8PwmRange;
// CLK_1024  FAST_PWM OC0_MODE
TCCR0A = _BV(WGM00) | _BV(WGM01) | _BV(COM0A1);
TCCR0B =  _BV(CS02) | _BV(CS00); // 1024
//16Mhz / 1024 / 256 = ~ 61 Herz 
DDRD |= _BV(PD6); //OC0A
};


//********************************
uint16_t ADC_10Bit(uint8_t u8Chan)
//********************************
/*
Externe 5V REF--Dreaht von AVCC zu 5V+
// PRESCALER 3,6864 / 32 = 115200 50-200Khz gewnscht
*/
{
DDRC &= ~_BV(u8Chan);  //Portx Channel Bit Input  
PORTC &= ~_BV(u8Chan); //Pullup abschalten
//Activate ADC with Prescaler 16 --> 16Mhz/128 = 125 khz
ADMUX = (1<< REFS0) | u8Chan;    //AVCC und AREF mit C an GND
ADCSRA = _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1) |_BV(ADPS0);  
//ADMUX = u8Chan | (1<<REFS0);
_delay_us(10);
ADCSRA |= _BV(ADSC);  //Start conversion 
while( ADCSRA & _BV(ADSC) ) // wait until converstion completed
	{
	//~150S
	wdt_reset();	//WATCHDOG!
	};	   
return ADC;// get converted value
};


#define ENABLE_DIR DDRC
#define ENABLE_PORT PORTC
#define ENABLE_BIT (1<<0)
#define ENABLE_INIT() 	(ENABLE_DIR |= ENABLE_BIT);
#define SET_ENABLE()	(ENABLE_PORT |= ENABLE_BIT)
#define CLR_ENABLE()	(ENABLE_PORT &= ~ENABLE_BIT)

// *************
int main(void)
// *************
{

uint8_t u8Val;
uint8_t u8NN;
uint16_t u16NN;

uint16_t u16Value;
uint16_t u16Pars;

//ENABLE_DIR |= ENABLE_BIT;
//DDRB |= LED1_BLINK;	     //Bit 0 von PORTC als Ausgang

wdt_reset();   //WATCHDOG!
wdt_enable(5); //500mS Totmannknopf

uart_init( UART_BAUD_SELECT_DOUBLE_SPEED( UART_BAUD_RATE, F_CPU) );
sei(); 
ResetRxBuff();

//TickCounter Starten
TCCR2B = (1 << CS22) | (1 << CS21);	//Vorteiler //256 einschalten
TIMSK2 = (1 << TOIE2);	//Interrrupt aktivieren

sei();
uart_puts_p( PSTR( HARDWARE ));uart_putc(13); 
uart_puts_p(  PSTR( SOFTWARE ) );uart_putc(13);
uart_puts_p( PSTR( DATUM )  );uart_putc(13); 
uart_puts_p( PSTR( AUTOR ) );uart_putc(13); 
uart_puts_p( PSTR( IHAVENOBRAIN ) );uart_putc(13); 
uart_puts_p( PSTR( USEYOURS ) );uart_putc(13); 

gu8Status |= STATUS_TRIGGER_OUTPUT;

//Timer0_PWM( 23 );

//ENABLE_INIT();
//SET_ENABLE();


u8NN = 0;
for(;;)  //ENSLESS
	{
	sei();
	wdt_reset();	//WATCHDOG!
	
	
	u16NN++;
	if(u16NN > 4095) 
		{
		u16NN=0;
		};
		
	//u16Value= 2048 + 2047 * sin( 2.0 * M_PI * (u16NN/4096.0) );	
	//WriteMAX543( u16Value );
		
	if( gu8Status & STATUS_TICK_EVENT )
		{
		gu8Status &= ~STATUS_TICK_EVENT;				
		
		#define USED_LED_CNT 16
		u8NN++;
		u8NN %= USED_LED_CNT;
		
		u16Pars = (uint16_t)(8 + 8 * sin( 2 * M_PI * ( u8NN / 16.0) ));		  
		u16Value = (1UL << u16Pars);	
		OUT_74595( u16Value );
			
		
		if( gu8Status & STATUS_TRIGGER_OUTPUT ) //TO/TF
			{
			
			ultoa( u16Pars , gcaStr, 10 );
			strcat_P( gcaStr, PSTR("   ") );
			
			strcat( gcaStr, ultoa( u16Value , gcaNumStr, 10 ) );
			strcat_P( gcaStr, PSTR("   ") );
			
			strcat( gcaStr, ultoa( u8NN , gcaNumStr, 10 ) );

			uart_puts_p( PSTR("POS=") );			
			uart_puts( gcaStr );			
			uart_putc( 13 ); //CR
			};
		};
//Toggle_Bit(1);
	};
}; // main
