

#include "ds1302.h"

//https://www.mikrocontroller.net/mc-project/Pages/AVR/Programme/Programme.html

UHRDATA_T gsUhrData;

unsigned char read_byte(unsigned char command)
{
	RTC_DDR = (1<<RTC_RST) | (1<<RTC_CLK);
	RTC_PORT |= (1<<RTC_RST);

	ds1302_write (0x81 | (command<<1));
	unsigned char temp = ds1302_read ();

	RTC_PORT &= ~(1<<RTC_RST);

	return temp;
}

void write_byte(unsigned char command, unsigned char byte)
{
	RTC_DDR = (1<<RTC_RST) | (1<<RTC_CLK);
	RTC_PORT |= (1<<RTC_RST);

	ds1302_write (0x80 | (command<<1));
	ds1302_write (byte);

	RTC_PORT &= ~(1<<RTC_RST);
}


void ds1302_write (unsigned char byte)
{
	unsigned char i;

	RTC_DDR = (1<<RTC_IO) | (1<<RTC_RST) | (1<<RTC_CLK);

	for(i = 0; i < 8; i++)
	{
		RTC_PORT &= ~(1<<RTC_IO);

		if(byte & (1<<i))
			RTC_PORT |= (1<<RTC_IO);

		RTC_PORT |= (1<<RTC_CLK);
		delay_us(2);
		RTC_PORT &= ~(1<<RTC_CLK);
		delay_us(2);
	}
}

unsigned char ds1302_read (void)
{
	unsigned char byte = 0;

	RTC_PORT &= ~(1<<RTC_IO);
	RTC_DDR &= ~(1<<RTC_IO);

	for (unsigned char i=0; i<8; i++)
	{
		if(RTC_PIN & (1<<RTC_IO))
			byte |= (1<<i);

		RTC_PORT |= (1<<RTC_CLK);
		delay_us(2);
		RTC_PORT &= ~(1<<RTC_CLK);
		delay_us(2);
	}
	return byte;
}


void ds1302_get_data (void)
{
	unsigned char temp = 0;
	/*disable write-protection*/
	write_byte (CONTROL, 0);
	/*read seconds*/
	temp = read_byte (SEC);
	gsUhrData.sec_l = (temp & 0x0F) + 48;
	gsUhrData.sec_h = ((temp & 0x70) >> 4) +48;
	/*read minutes*/
	temp = read_byte (MIN);
	gsUhrData.min_l = (temp & 0x0F) + 48;
	gsUhrData.min_h = ((temp & 0x70) >> 4) + 48;
	/*read hours*/
	temp = read_byte (HR);
	gsUhrData.hr_l = (temp & 0x0F) + 48;
	gsUhrData.hr_h = ((temp & 0x30) >> 4) + 48;
	/*read date*/
	temp = read_byte (DATE);
	gsUhrData.date_l = (temp & 0x0F) + 48;
	gsUhrData.date_h = ((temp & 0x30) >> 4) + 48;
	/*read month*/
	temp = read_byte (MONTH);
	gsUhrData.month_l = (temp & 0x0F) + 48;
	gsUhrData.month_h = ((temp & 0x30) >> 4) + 48;
	/*read day*/
	temp = read_byte (DAY);			
	gsUhrData.day = (temp & 0x03);
	/*read year*/
	temp = read_byte (YEAR);
	gsUhrData.year_l = (temp & 0x0F) + 48;
	gsUhrData.year_h = ((temp & 0xF0) >> 4) +48;
	/*enable write-protection*/
	write_byte (CONTROL, 0x80);
}


void ds1302_set_data (void)
{
	/*disable write-protection*/
	write_byte (CONTROL, 0);
	/*write seconds*/
	write_byte (SEC, (gsUhrData.sec_l) + ((gsUhrData.sec_h) << 4));
	/*write minutes*/
	write_byte (MIN, (gsUhrData.min_l) + ((gsUhrData.min_h) << 4));
	/*write hours*/
	write_byte (HR, (gsUhrData.hr_l) + ((gsUhrData.hr_h) << 4));
	/*write date*/
	write_byte (DATE, (gsUhrData.date_l) + ((gsUhrData.date_h) << 4));
	/*write month*/
	write_byte (MONTH, (gsUhrData.month_l) + ((gsUhrData.month_h) << 4));
	/*write day*/
	write_byte (DAY, gsUhrData.day);
	/*write year*/
	write_byte (YEAR, (gsUhrData.year_l) + ((gsUhrData.year_h) << 4));
	/*enable write-protection*/
	write_byte (CONTROL, 0x80);
}

